/** /Routing/Error
 *
 * @author Alex Malotky
 */
import Context from "../Context";
interface FormatedError {
    status: number | string;
    message: string;
}
export type FormatedErrorHandler = (error: string, context: Context) => Promise<void> | void;
export type DefaultErrorHandler = (error: FormatedError, context: Context) => Promise<void> | void;
/** Error Routing
 *
 * Seperates Error Handlers based on status codes
 * Falls back to a default handler if one is set.
 */
export default class ErrorRouting extends Map<number | string, FormatedErrorHandler> {
    private _default;
    /** Format Error
     *
     * @param {any} error
     * @returns {InternalError}
     */
    private static formatError;
    /** Internal Handle Error
     *
     * Performs error handler if it exists
     *
     * If handler throws new error, will double check if error handler can handle that error.
     *
     * @param {InternalError} error
     * @param {Context} context
     * @returns {Promise<InternalError>}
     */
    private _handle;
    /** Handle Error
     *
     * Will check for handlers based on error status.
     *
     * If not no response is sent, will check for defualt error handler.
     *
     * @param {any} error
     * @param {Context} context
     */
    handle(error: any, context: Context): Promise<void>;
    /** Set Error Handler
     *
     * If no status is set, will set to default/fallback error handler.
     *
     * @param {string|number} status
     * @param {FormatedErrorHandler} handler
     */
    set(status: string | number, handler: FormatedErrorHandler): this;
    set(handler: DefaultErrorHandler): this;
}
export {};
