"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Router_1 = __importDefault(require("./Router"));
/** Routing Class
 *
 * Top Level Wrapper around Router class that
 * handles errors.
 */
class Routing extends Router_1.default {
    /** Routing Constructor
     *
     */
    constructor() {
        super("");
        this._errors.set((err, ctx) => {
            const status = isNaN(err.status) ? 500 : Number(err.status);
            const message = String(err.message);
            ctx.status(status);
            if (ctx.expectsRender()) {
                ctx.render({
                    body: {
                        error: message
                    }
                });
            }
            else {
                ctx.write(message);
            }
            ctx.end();
        });
    }
    /** Routing Handler Wrappper
     *
     *
     * @param {Context} context
     * @returns {Promise<Response>}
     */
    async route(context) {
        await this.handle(context);
        return await context.flush();
    }
    /** Handle Routing Override
     *
     * @param {Context} context
     * @returns {Promise<void>}
     */
    async handle(context) {
        try {
            for (const { name, layer } of this._methods) {
                if (name === "MIDDLEWARE") {
                    await layer.handle(context);
                    if (context.response.commited())
                        return;
                }
                else if (name === context.method || name === "ALL") {
                    await layer.handle(context);
                    if (context.response.commited())
                        return;
                }
            }
            throw 404;
        }
        catch (err) {
            this._errors.handle(err, context);
        }
    }
}
exports.default = Routing;
