"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.objectify = exports.ObjectName = void 0;
const Type_1 = require("./Type");
const Empty_1 = require("./Type/Empty");
//Object Format Name
exports.ObjectName = "Object";
/** Object Validator
 *
 */
class ObjectValidator extends Type_1.TypeValidator {
    constructor(format, value) {
        if (value) {
            if (typeof value !== "object")
                throw new TypeError("Default value is not an object!");
            value = buildObject(format, value);
        }
        super(exports.ObjectName, formatObjectGenerator(format, value));
        this._props = format;
        this._default = value;
    }
    simplify(value) {
        return JSON.stringify(value);
    }
    clone(value) {
        if (typeof value === "boolean") {
            if (value) {
                value = this._default;
            }
            else {
                value = undefined;
            }
        }
        return new ObjectValidator(this._props, value);
    }
}
exports.default = ObjectValidator;
/** Format Object Generator
 *
 * @param {Object} props
 * @returns {Function}
 */
function formatObjectGenerator(props, ifEmpty) {
    /** Format Object
     *
     * @param {unknown} input
     * @returns {Object}
     */
    return function formatObject(input) {
        return (0, Empty_1.emptyHandler)(input, (value) => buildObject(props, objectify(value), ifEmpty), ifEmpty);
    };
}
/** Build Object
 *
 * @param {ObjectProperties} props
 * @param {Object} value
 * @returns {Object}
 */
function buildObject(props, value, defaultValue = {}) {
    const output = {};
    const expected = Object.getOwnPropertyNames(props);
    for (const name in value) {
        const index = expected.indexOf(name);
        if (index === -1)
            throw new Error(`Unexpected value occured at ${name}!`);
        expected.splice(index, 1);
        try {
            output[name] = props[name].run(value[name]);
        }
        catch (e) {
            throw new Error(`${e.message || String(e)} at ${name}`);
        }
    }
    for (const name of expected) {
        try {
            output[name] = props[name].run(defaultValue[name]);
        }
        catch (e) {
            throw new Error(`${e.message || String(e)} at ${name}`);
        }
    }
    return output;
}
/** Object Objectifier
 *
 * @param {unkown} value
 * @returns {Record<string, unknown>}
 */
function objectify(value) {
    switch (typeof value) {
        case "string":
            return objectify(JSON.parse(value));
        case "object":
            if (Array.isArray(value))
                throw new TypeError(`Invalid object type: 'Array'!`);
            if (value === null)
                throw new TypeError(`Invalid object type: 'null'!`);
            return value;
        default:
            throw new TypeError(`Invalid object type: '${typeof value}'!`);
    }
}
exports.objectify = objectify;
